@@ -7,6 +7,7 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::{read_prefix_map_from_disk, UsedRecipientSaps};
+use crate::messaging::signature_aggregator::{Error as AggregatorError, SignatureAggregator};
 use crate::messaging::{
     system::{
         JoinRejectionReason, JoinRequest, JoinResponse, ResourceProofResponse, SectionAuth,
@@ -30,6 +31,7 @@ use backoff::{backoff::Backoff, ExponentialBackoff};
 use bls::PublicKey as BlsPublicKey;
 use futures::future;
 use resource_proof::ResourceProof;
+use tokio::time::sleep;
 use tokio::{sync::mpsc, time::Duration};
 use tracing::Instrument;
 use xor_name::Prefix;
@@ -69,6 +71,8 @@ struct Join<'a> {
     node: Node,
     prefix: Prefix,
     prefix_map: NetworkPrefixMap,
+    signature_aggregator: SignatureAggregator,
+    node_state_serialized: Option<Vec<u8>>,
     backoff: ExponentialBackoff,
 }
 
@@ -85,6 +89,8 @@ impl<'a> Join<'a> {
             node,
             prefix: Prefix::default(),
             prefix_map,
+            signature_aggregator: SignatureAggregator::new(),
+            node_state_serialized: None,
             backoff: ExponentialBackoff {
                 initial_interval: Duration::from_millis(50),
                 max_interval: Duration::from_millis(750),
@@ -135,9 +141,10 @@ impl<'a> Join<'a> {
         let join_request = JoinRequest {
             section_key,
             resource_proof_response: None,
+            aggregated: None,
         };
 
-        self.send_join_requests(join_request, &recipients, section_key, false)
+        self.send_join_requests(join_request.clone(), &recipients, section_key, false)
             .await?;
 
         // Avoid sending more than one duplicated request (with same SectionKey) to the same peer.
@@ -145,7 +152,6 @@ impl<'a> Join<'a> {
 
         loop {
             let (response, sender) = self.receive_join_response().await?;
-
             match response {
                 JoinResponse::Rejected(JoinRejectionReason::NodeNotReachable(addr)) => {
                     error!(
@@ -158,6 +164,12 @@ impl<'a> Join<'a> {
                     error!("Network is set to not taking any new joining node, try join later.");
                     return Err(Error::TryJoinLater);
                 }
+                JoinResponse::Rejected(JoinRejectionReason::DKGUnderway) => {
+                    error!("The Section we are trying to join is going through DKG, backing off and retrying again");
+                    self.send_join_requests(join_request.clone(), &recipients, section_key, true)
+                        .await?;
+                    continue;
+                }
                 JoinResponse::Approval {
                     section_auth,
                     genesis_key,
@@ -184,15 +196,71 @@ impl<'a> Join<'a> {
                     );
 
                     // Building our network knowledge instance will validate SAP and section chain.
+                    let section_auth = section_auth.into_authed_state();
+
+                    // Include the sender's connection in our initial knowledge
+                    section_auth.merge_connections([&sender]).await;
+
                     let network_knowledge = NetworkKnowledge::new(
                         genesis_key,
                         section_chain,
-                        section_auth.into_authed_state(),
+                        section_auth,
                         Some(self.prefix_map),
                     )?;
 
                     return Ok((self.node, network_knowledge));
                 }
+                JoinResponse::ApprovalShare {
+                    node_state,
+                    sig_share,
+                } => {
+                    let serialized_details =
+                        if let Some(node_state_serialized) = &self.node_state_serialized {
+                            node_state_serialized.clone()
+                        } else {
+                            let node_state_serialized = bincode::serialize(&node_state)?;
+                            self.node_state_serialized = Some(node_state_serialized.clone());
+                            node_state_serialized
+                        };
+
+                    info!("Aggregating received ApprovalShare from {:?}", sender);
+                    match self
+                        .signature_aggregator
+                        .add(&serialized_details, sig_share.clone())
+                        .await
+                    {
+                        Ok(sig) => {
+                            info!("Successfully aggregated ApprovalShares for joining the network");
+
+                            let section_key = sig_share.public_key_set.public_key();
+                            let auth = SectionAuth {
+                                value: node_state,
+                                sig,
+                            };
+                            let join_req = JoinRequest {
+                                section_key,
+                                resource_proof_response: None,
+                                aggregated: Some(auth),
+                            };
+                            let name = self.node.name();
+                            let recipients: Vec<Peer> = if let Some(signed_sap) =
+                                self.prefix_map.closest_or_opposite(&name, None)
+                            {
+                                signed_sap.value.elders().cloned().collect()
+                            } else {
+                                warn!("cannot find recipients to send aggregated JoinApproval");
+                                continue;
+                            };
+                            trace!("Sending aggregated JoinRequest to {:?}", recipients);
+                            // Resend the JoinRequest now that we have collected enough ApprovalShares from the Elders
+                            self.send_join_requests(join_req, &recipients, section_key, false)
+                                .await?;
+                            continue;
+                        }
+                        Err(AggregatorError::NotEnoughShares) => continue,
+                        _ => return Err(Error::InvalidSignatureShare),
+                    }
+                }
                 JoinResponse::Retry {
                     section_auth,
                     section_signed,
@@ -241,7 +309,7 @@ impl<'a> Join<'a> {
 
                     // if it's not a new SAP, ignore response unless the expected age is different.
                     if self.node.age() != expected_age
-                        && (self.node.age() > expected_age || self.node.age() == MIN_ADULT_AGE)
+                        && (self.node.age() > expected_age || self.node.age() <= MIN_ADULT_AGE)
                     {
                         // adjust our joining age to the expected by the network
                         trace!(
@@ -249,20 +317,19 @@ impl<'a> Join<'a> {
                             self.node.age(),
                             expected_age
                         );
-
                         // The expected_age is a sequence of 98, 96, 94, 92, ...
-                        // The expected_prefix is deduced from it.
+                        // The prefix is deduced from the age's bits.
                         let mut cur_age = expected_age / 2;
-                        let mut expected_prefix = Prefix::default();
+                        let mut new_prefix = Prefix::default();
                         while cur_age > 0 {
-                            let remain_age = cur_age / 2;
-                            expected_prefix = expected_prefix.pushed(cur_age != (remain_age * 2));
-                            cur_age = remain_age;
+                            let push_prefix_0 = cur_age % 2 == 1;
+                            new_prefix = new_prefix.pushed(push_prefix_0);
+                            cur_age /= 2;
                         }
-                        trace!("Name shall have the prefix of {:?}", expected_prefix);
+                        trace!("Name shall have the prefix of {:?}", new_prefix);
 
                         let new_keypair =
-                            ed25519::gen_keypair(&expected_prefix.range_inclusive(), expected_age);
+                            ed25519::gen_keypair(&new_prefix.range_inclusive(), expected_age);
                         let new_name = ed25519::name(&new_keypair.public);
 
                         info!("Setting Node name to {} (age {})", new_name, expected_age);
@@ -283,6 +350,7 @@ impl<'a> Join<'a> {
                     let join_request = JoinRequest {
                         section_key,
                         resource_proof_response: None,
+                        aggregated: None,
                     };
 
                     section_auth
@@ -341,6 +409,7 @@ impl<'a> Join<'a> {
                     let join_request = JoinRequest {
                         section_key,
                         resource_proof_response: None,
+                        aggregated: None,
                     };
 
                     section_auth
@@ -369,6 +438,7 @@ impl<'a> Join<'a> {
                             nonce,
                             nonce_signature,
                         }),
+                        aggregated: None,
                     };
                     let recipients = &[sender];
                     self.send_join_requests(join_request, recipients, section_key, false)
@@ -391,11 +461,11 @@ impl<'a> Join<'a> {
             let next_wait = self.backoff.next_backoff();
 
             if let Some(wait) = next_wait {
-                tokio::time::sleep(wait).await;
+                sleep(wait).await;
             } else {
                 error!("Waiting before attempting to join again");
 
-                tokio::time::sleep(self.backoff.max_interval).await;
+                sleep(self.backoff.max_interval).await;
                 self.backoff.reset();
             }
         }
