@@ -13,7 +13,6 @@ mod capacity;
 mod chunk_records;
 mod chunk_store;
 mod comm;
-mod connected_peers;
 mod connectivity;
 mod delivery_group;
 mod liveness_tracking;
@@ -35,7 +34,8 @@ pub(crate) use register_storage::RegisterStorage;
 use self::split_barrier::SplitBarrier;
 use crate::dbs::UsedSpace;
 use crate::messaging::signature_aggregator::SignatureAggregator;
-use crate::messaging::system::SystemMsg;
+use crate::messaging::system::{DkgSessionId, SystemMsg};
+use crate::messaging::{AuthorityProof, SectionAuth};
 use crate::routing::{
     dkg::DkgVoter,
     error::Result,
@@ -52,6 +52,8 @@ use capacity::Capacity;
 use itertools::Itertools;
 use liveness_tracking::Liveness;
 use resource_proof::ResourceProof;
+use std::collections::HashMap;
+use std::net::SocketAddr;
 use std::{
     collections::{BTreeMap, BTreeSet},
     path::PathBuf,
@@ -82,6 +84,13 @@ const CHUNK_QUERY_TIMEOUT: Duration = Duration::from_secs(60 * 5 /* 5 mins */);
 pub(crate) type AeBackoffCache =
     Arc<RwLock<LRUCache<(Peer, ExponentialBackoff), BACKOFF_CACHE_LIMIT>>>;
 
+#[derive(Clone)]
+pub(crate) struct DkgSessionInfo {
+    pub(crate) prefix: Prefix,
+    pub(crate) elders: BTreeMap<XorName, SocketAddr>,
+    pub(crate) authority: AuthorityProof<SectionAuth>,
+}
+
 // State + logic of a routing node.
 pub(crate) struct Core {
     pub(crate) comm: Comm,
@@ -91,8 +100,11 @@ pub(crate) struct Core {
     message_aggregator: SignatureAggregator,
     proposal_aggregator: SignatureAggregator,
     split_barrier: Arc<RwLock<SplitBarrier>>,
+    dkg_sessions: Arc<RwLock<HashMap<DkgSessionId, DkgSessionInfo>>>,
     // Voter for Dkg
     dkg_voter: DkgVoter,
+    // Boolean to represent ongoing DKG
+    is_dkg_underway: Arc<RwLock<bool>>,
     relocate_state: Arc<RwLock<Option<RelocateState>>>,
     pub(super) event_tx: mpsc::Sender<Event>,
     joins_allowed: Arc<RwLock<bool>>,
@@ -137,10 +149,12 @@ impl Core {
             node: Arc::new(RwLock::new(node)),
             network_knowledge,
             section_keys_provider,
+            dkg_sessions: Arc::new(RwLock::new(HashMap::default())),
             proposal_aggregator: SignatureAggregator::default(),
             split_barrier: Arc::new(RwLock::new(SplitBarrier::new())),
             message_aggregator: SignatureAggregator::default(),
             dkg_voter: DkgVoter::default(),
+            is_dkg_underway: Arc::new(RwLock::new(false)),
             relocate_state: Arc::new(RwLock::new(None)),
             event_tx,
             joins_allowed: Arc::new(RwLock::new(true)),
