@@ -31,9 +31,6 @@ impl Core {
     ) -> Result<Vec<Command>> {
         debug!("handle agreement on {:?}", proposal);
         match proposal {
-            Proposal::Online { node_state, .. } => {
-                self.handle_online_agreement(node_state, sig).await
-            }
             Proposal::Offline(node_state) => self.handle_offline_agreement(node_state, sig).await,
             Proposal::SectionInfo(section_auth) => {
                 self.handle_section_info_agreement(section_auth, sig).await
@@ -49,7 +46,7 @@ impl Core {
         }
     }
 
-    async fn handle_online_agreement(
+    pub(crate) async fn handle_online_agreement(
         &self,
         new_info: NodeState,
         sig: KeyedSig,
@@ -129,6 +126,8 @@ impl Core {
         commands.extend(result);
         commands.extend(self.send_node_approval(new_info).await);
 
+        info!("Commands in queue for Accepting node {:?}", commands);
+
         self.print_network_stats().await;
 
         Ok(commands)
@@ -274,6 +273,7 @@ impl Core {
                     .key_share(&signed_sap.section_key())
                     .await
                     .is_ok();
+            trace!("switch_to_new_sap {:?}", switch_to_new_sap);
 
             // Let's update our network knowledge, including our
             // section SAP and chain if the new SAP's prefix matches our name
@@ -309,6 +309,8 @@ impl Core {
             }
         }
 
+        *self.is_dkg_underway.write().await = false;
+
         info!(
             "Prefixes we know about: {:?}",
             self.network_knowledge.prefix_map()
